@@ -183,11 +183,19 @@ static int S_render_node(cmark_html_renderer *renderer, cmark_node *node,
         first_tag += 1;
       }
 
-      cmark_strbuf_puts(html, "<pre");
-      cmark_html_render_sourcepos(node, html, options);
-      cmark_strbuf_puts(html, "><code class=\"language-");
-      escape_html(html, node->as.code.info.data, first_tag);
-      cmark_strbuf_puts(html, "\">");
+      if (options & CMARK_OPT_GITHUB_PRE_LANG) {
+        cmark_strbuf_puts(html, "<pre");
+        cmark_html_render_sourcepos(node, html, options);
+        cmark_strbuf_puts(html, " lang=\"");
+        escape_html(html, node->as.code.info.data, first_tag);
+        cmark_strbuf_puts(html, "\"><code>");
+      } else {
+        cmark_strbuf_puts(html, "<pre");
+        cmark_html_render_sourcepos(node, html, options);
+        cmark_strbuf_puts(html, "><code class=\"language-");
+        escape_html(html, node->as.code.info.data, first_tag);
+        cmark_strbuf_puts(html, "\">");
+      }
     }
 
     escape_html(html, node->as.code.literal.data, node->as.code.literal.len);
