@@ -12,11 +12,13 @@ struct cmark_syntax_extension {
   cmark_llist                   * special_inline_chars;
   char                          * name;
   void                          * priv;
+  bool                            emphasis;
   cmark_free_func                 free_function;
   cmark_get_type_string_func      get_type_string_func;
   cmark_can_contain_func          can_contain_func;
   cmark_contains_inlines_func     contains_inlines_func;
   cmark_common_render_func        commonmark_render_func;
+  cmark_common_render_func        plaintext_render_func;
   cmark_common_render_func        latex_render_func;
   cmark_common_render_func        man_render_func;
   cmark_html_render_func          html_render_func;
